import java.io.*; 
import ij.*; 
import ij.io.*; 
import ij.gui.*; 
import ij.util.Tools;
import ij.plugin.*; 
import ij.process.*; 
import java.util.*;
import ij.plugin.filter.*; 

public class ESRIASCIIGrid_Writer implements PlugInFilter {

    ImagePlus img;
    ImageProcessor ip;

    public ESRIASCIIGrid_Writer() {
        img = null;
        ip = null;
    }

    public int setup(String arg, ImagePlus imp) {
        img = imp;
        return DOES_32;
    }

    public void run(ImageProcessor ip) {
        SaveDialog sd = new SaveDialog("Save ESRI ASCII Grid", "grid", ".asc");
        String directory = sd.getDirectory();
        String fileName = sd.getFileName();
        if (fileName == null) {
            return;
        }
        IJ.showStatus("Saving: " + directory + fileName);
        this.ip = ip;
        write(directory, fileName);
    }
	
	private void writeHeader(BufferedWriter bw) throws java.io.IOException {
		String lineSeparator = System.getProperty("line.separator");
        bw.write("ncols " + img.getWidth() + lineSeparator);
        bw.write("nrows " + img.getHeight() + lineSeparator);
        bw.write("xllcorner " + 0 + lineSeparator);
        bw.write("yllcorner " + 0 + lineSeparator);
        bw.write("cellsize " + 100 + lineSeparator);
        bw.write("nodata_value " + -Float.MAX_VALUE + lineSeparator);
	}

    protected void write(String dir, String filename) {
        try {
			String lineSeparator = System.getProperty("line.separator");
            BufferedWriter bw = new BufferedWriter(new FileWriter(dir + filename));
			writeHeader(bw);
            float[] pixels = (float[]) ip.getPixels();
            int offset, pos;
            for (int j = 0; j < img.getHeight(); j++) {
                offset = j * img.getWidth();
                for (int i = 0; i < img.getWidth(); i++) {
                    pos = offset + i;
                    bw.write(new Float(pixels[pos]).toString());
                    if (i < img.getWidth() - 1) {
                        bw.write(" ");
                    }
                }
                if (j < img.getHeight() - 1) {
                    bw.write(lineSeparator);
                }
            }
            bw.close();
        } catch (Exception e) {
            IJ.error("ESRI ASCII Grid Writer", e.getMessage());
            return;
        }
    }
}
